@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -73,7 +73,6 @@
  *
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  * @author Martin Pala <martinp@tildeslash.com>
- * @version \$Id: event.c,v 1.99 2009/05/28 21:35:29 martinp Exp $
  * @file
  */
 
@@ -148,7 +147,6 @@ void Event_post(Service_T service, long id, short state, EventAction_T action, c
     e->id = id;
     gettimeofday(&e->collected, NULL);
     e->source = xstrdup(service->name);
-    e->group = service->group ? xstrdup(service->group) : xstrdup("");
     e->mode = service->mode;
     e->type = service->type;
     e->state = STATE_INIT;
@@ -206,7 +204,6 @@ void Event_post(Service_T service, long id, short state, EventAction_T action, c
       e->id = id;
       gettimeofday(&e->collected, NULL);
       e->source = xstrdup(service->name);
-      e->group = service->group ? xstrdup(service->group) : xstrdup("");
       e->mode = service->mode;
       e->type = service->type;
       e->state = STATE_INIT;
@@ -269,17 +266,6 @@ char *Event_get_source_name(Event_T E) {
 }
 
 
-/**
- * Get the group name of the service where the event orginated
- * @param E An event object
- * @return The group name of the service where the event orginated
- */
-char *Event_get_source_group(Event_T E) {
-  ASSERT(E);
-  return (E->group);
-}
-
-
 /**
  * Get the service type of the service where the event orginated
  * @param E An event object
@@ -350,7 +336,7 @@ short Event_check_state(Event_T E, short S) {
   }
 
   /* the internal instance and action events are handled as changed any time since we need to deliver alert whenever it occurs */
-  if (E->id == EVENT_INSTANCE || E->id == EVENT_ACTION || (count >= action->count && S != E->state))
+  if (E->id == EVENT_INSTANCE || E->id == EVENT_ACTION || (count >= action->count && (S != E->state || S == STATE_CHANGED)))
     return TRUE;
   
   return FALSE;
@@ -541,19 +527,15 @@ void Event_queue_process() {
       if (!(e->source = File_readQueue(file, &size)))
         goto error4;
 
-      /* read group */
-      if (!(e->group = File_readQueue(file, &size)))
-        goto error5;
-
       /* read message */
       if (!(e->message = File_readQueue(file, &size)))
-        goto error6;
+        goto error5;
 
       /* read event action */
       if (!(action = File_readQueue(file, &size)))
-        goto error7;
+        goto error6;
       if (size != sizeof(short))
-        goto error8;
+        goto error7;
       a->id = *action;
       if (e->state == STATE_FAILED)
         ea->failed = a;
@@ -605,12 +587,10 @@ void Event_queue_process() {
         Event_queue_update(e, file_name);
       }
 
-error8:
-      FREE(action);
 error7:
-      FREE(e->message);
+      FREE(action);
 error6:
-      FREE(e->group);
+      FREE(e->message);
 error5:
       FREE(e->source);
 error4:
@@ -732,7 +712,7 @@ static void handle_action(Event_T E, Action_T A) {
     spawn(s, A->exec, Event_get_description(E));
     return;
   } else {
-    if (s->def_timeout && (A->id == ACTION_START || A->id == ACTION_RESTART))
+    if (s->actionratelist && (A->id == ACTION_START || A->id == ACTION_RESTART))
       s->nstart++;
 
     if (s->mode == MODE_PASSIVE && (A->id == ACTION_START || A->id == ACTION_STOP  || A->id == ACTION_RESTART))
@@ -793,10 +773,6 @@ static void Event_queue_add(Event_T E) {
   if (!(rv = File_writeQueue(file, E->source, E->source ? strlen(E->source)+1 : 0)))
     goto error;
 
-  /* write group */
-  if (!(rv = File_writeQueue(file, E->group, E->group ? strlen(E->group)+1 : 0)))
-    goto error;
-
   /* write message */
   if (!(rv = File_writeQueue(file, E->message, E->message ? strlen(E->message)+1 : 0)))
     goto error;
@@ -865,10 +841,6 @@ static void Event_queue_update(Event_T E, const char *file_name) {
   if (!(rv = File_writeQueue(file, E->source, E->source ? strlen(E->source)+1 : 0)))
     goto error;
 
-  /* write group */
-  if (!(rv = File_writeQueue(file, E->group, E->group ? strlen(E->group)+1 : 0)))
-    goto error;
-
   /* write message */
   if (!(rv = File_writeQueue(file, E->message, E->message ? strlen(E->message)+1 : 0)))
     goto error;
