@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -81,7 +81,6 @@ extern EventTable_T Event_Table[];
  * 
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  * @author Martin Pala, <martinp@tildeslash.com>
- * @version \$Id: event.h,v 1.37 2009/04/19 20:13:57 martinp Exp $
  * @file
  */
 
@@ -112,14 +111,6 @@ Service_T Event_get_source(Event_T E);
 char *Event_get_source_name(Event_T E);
 
 
-/**
- * Get the group name of the service where the event orginated
- * @param E An event object
- * @return The group name of the service where the event orginated
- */
-char *Event_get_source_group(Event_T E);
-
-
 /**
  * Get the service type of the service where the event orginated
  * @param E An event object
