@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -158,8 +158,6 @@ static char *get_service_status_text(Service_T);
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: cervlet.c,v 1.260 2009/05/28 07:13:12 hauk Exp $
- *
  *  @file
  */
 
@@ -257,7 +255,6 @@ static void is_monit_running(HttpRequest req, HttpResponse res) {
     
 
 static void do_home(HttpRequest req, HttpResponse res) {
-
   char *uptime= Util_getUptime(Util_getProcessUptime(Run.pidfile), "&nbsp;");
  
   HEAD("", Run.polltime)
@@ -288,7 +285,6 @@ static void do_home(HttpRequest req, HttpResponse res) {
   do_home_host(req, res);
   
   FOOT
-      
 }
 
 
@@ -300,7 +296,7 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "monit " VERSION "</a></center></h1>");
   out_print(res,
     "<ul>"
-    "<li style='padding-bottom:10px;'>Copyright &copy; 2000-2009 by <a "
+    "<li style='padding-bottom:10px;'>Copyright &copy; 2000-2010 by <a "
     "href=\"http://tildeslash.com/\">Tildeslash Ltd"
     "</a>. All Rights Reserved.</li>"
     "<li>Portions of this software are copyright &copy; 1995, 1996 "
@@ -314,14 +310,10 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
     "<a href='http://www.gnu.org/licenses/gpl.html'>GNU "
     "General Public License</a> for more details.</p>"
-    "<p style='border-top:1px solid #ccc; padding-top:1em;'>"
-    "Running Monit on more than one server? Use <a href='http://mmonit.com/'>"
-    "M/Monit</a> to manage all your Monit instances.</p>"
-    "<p style='font-weight: bold;'>Please consider making a <a href=\"http://mmonit.com/monit/donate/\">"
+    "<p style='font-weight: bold; text-align:center; padding-top:30px;'>Please consider making a <a href=\"http://mmonit.com/monit/donate/\">"
     "donation</a> to support our work with Monit.</p>");
   out_print(res,
-    "<center><p style='padding-top:20px;'>[Click <a href=\".\">here</a> to "
-    "go back to Monit]</p></body></html>");
+    "<center><p style='padding-top:20px;'>[<a href=\".\">Back to Monit</a>]</p></body></html>");
 
 }
 
@@ -401,8 +393,8 @@ static void do_runtime(HttpRequest req, HttpResponse res) {
   }
 
   if(Run.mailservers) {
-    out_print(res, "<tr><td>Mail server(s)</td><td>");
     MailServer_T mta= Run.mailservers;
+    out_print(res, "<tr><td>Mail server(s)</td><td>");
     for(mta= Run.mailservers; mta; mta= mta->next)
 	out_print(res, "%s:%d%s&nbsp;",
           mta->host, mta->port, mta->ssl.use_ssl?"(ssl)":"");
@@ -521,7 +513,7 @@ static void do_viewlog(HttpRequest req, HttpResponse res) {
   
   if(is_readonly(req)) {
     send_error(res, SC_FORBIDDEN,
-	       "You do not have sufficent privilegs to access this page");
+	       "You do not have sufficent privileges to access this page");
     return;
   }
   
@@ -587,8 +579,7 @@ static void do_viewlog(HttpRequest req, HttpResponse res) {
 
 
 static void handle_action(HttpRequest req, HttpResponse res) {
-
-  char *name= req->url;
+  char *name = req->url;
   const char *action;
   Service_T s;
   
@@ -600,8 +591,7 @@ static void handle_action(HttpRequest req, HttpResponse res) {
     const char *token = NULL;
 
     if(is_readonly(req)) {
-      send_error(res, SC_FORBIDDEN,
-        "You do not have sufficent privilegs to access this page");
+      send_error(res, SC_FORBIDDEN, "You do not have sufficent privileges to access this page");
       return;
     }
     if((s->doaction = Util_getAction(action)) == ACTION_IGNORE) {
@@ -609,18 +599,16 @@ static void handle_action(HttpRequest req, HttpResponse res) {
       return;
     }
     token = get_parameter(req, "token");
-    s->token = token?xstrdup(token):NULL;
+    s->token = token ? xstrdup(token) : NULL;
     LogDebug("%s service '%s' on user request\n", action, s->name);
     Run.doaction = TRUE; /* set the global flag */
     do_wakeupcall();
   }
   do_service(req, res, s);
-
 }
 
 
 static void handle_do_action(HttpRequest req, HttpResponse res) {
-
   Service_T s;
   int doaction = ACTION_IGNORE;
   const char *action = get_parameter(req, "action");
@@ -630,7 +618,7 @@ static void handle_do_action(HttpRequest req, HttpResponse res) {
     char *token = NULL;
 
     if(is_readonly(req)) {
-      send_error(res, SC_FORBIDDEN, "You do not have sufficent privilegs to access this page");
+      send_error(res, SC_FORBIDDEN, "You do not have sufficent privileges to access this page");
       return;
     }
     if((doaction = Util_getAction(action)) == ACTION_IGNORE) {
@@ -673,7 +661,6 @@ static void handle_do_action(HttpRequest req, HttpResponse res) {
     Run.doaction = TRUE; 
     do_wakeupcall();
   }
-
 }
 
 
@@ -684,7 +671,7 @@ static void handle_run(HttpRequest req, HttpResponse res) {
   if(action) {
     if(is_readonly(req)) {
       send_error(res, SC_FORBIDDEN,
-        "You do not have sufficent privilegs to access this page");
+        "You do not have sufficent privileges to access this page");
       return;
     }
     if(IS(action, "validate")) {
@@ -707,157 +694,143 @@ static void handle_run(HttpRequest req, HttpResponse res) {
 
 
 static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
-
   Dependant_T d;
+  ActionRate_T ar;
+  ServiceGroup_T sg;
+  ServiceGroupMember_T sgm;
   char *status;
   char time[STRLEN];
 
   ASSERT(s);
 
-  LOCK(s->mutex)
+  HEAD(s->name, Run.polltime)
 
-    HEAD(s->name, Run.polltime)
+  out_print(res,
+    "<p><br><h3>%s status</h3><br>"
+    "<table cellspacing=0 cellpadding=3 border=1 width=\"90%%\" style=\"border:1px solid #ccc;border-collapse:collapse;\">"
+    "<tr>"
+    "<td width=\"30%%\" bgcolor=\"#edf5ff\"><b>Parameter</b></td>"
+    "<td width=\"70%%\" bgcolor=\"#edf5ff\"><b>Value</b></td>"
+    "</tr>"
+    "<tr>"
+    "<td>Name</td>"
+    "<td>%s</td>"
+    "</tr>",
+    servicetypes[s->type],
+    s->name);
 
+  if(s->path && *s->path)
     out_print(res,
-      "<p><br><h3>%s status</h3><br>"
-      "<table cellspacing=0 cellpadding=3 border=1 width=\"90%%\" style=\"border:1px solid #ccc;border-collapse:collapse;\">"
       "<tr>"
-      "<td width=\"30%%\" bgcolor=\"#edf5ff\"><b>Parameter</b></td>"
-      "<td width=\"70%%\" bgcolor=\"#edf5ff\"><b>Value</b></td>"
-      "</tr>"
-      "<tr>"
-      "<td>Name</td>"
+      "<td>%s</td>"
       "<td>%s</td>"
       "</tr>",
-      servicetypes[s->type],
-      s->name);
+      pathnames[s->type],
+      s->path);
 
-    if(s->path && *s->path)
+  status= get_service_status_html(s);
+  out_print(res,
+    "<tr><td>Status</td><td>%s</td></tr>", status);
+  FREE(status);
+
+  for (sg = servicegrouplist; sg; sg = sg->next)
+    for (sgm = sg->members; sgm; sgm = sgm->next)
+      if (! strcasecmp(sgm->name, s->name))
+        out_print(res, "<tr><td>Group</td><td><font color='#0000ff'>%s</font></td></tr>", sg->name);
+
+  out_print(res,
+    "<tr><td>Monitoring mode</td><td>%s</td></tr>", modenames[s->mode]);
+
+  out_print(res,
+    "<tr><td>Monitoring status</td><td><font color='#ff8800'>%s</font></td></tr>", monitornames[s->monitor]);
+
+  for(d= s->dependantlist; d; d= d->next) {
+    if(d->dependant != NULL) {
       out_print(res,
-        "<tr>"
-        "<td>%s</td>"
-        "<td>%s</td>"
-        "</tr>",
-        pathnames[s->type],
-        s->path);
-
-    status= get_service_status_html(s);
-    out_print(res,
-      "<tr><td>Status</td><td>%s</td></tr>", status);
-    FREE(status);
-
-    if(s->group)
-      out_print(res,
-        "<tr><td>Group</td><td><font color='#0000ff'>%s</font></td></tr>",
-        s->group);
-
-    out_print(res,
-      "<tr><td>Monitoring mode</td><td>%s</td></tr>", modenames[s->mode]);
-
-    out_print(res,
-      "<tr><td>Monitoring status</td><td><font color='#ff8800'>%s"
-      "</font></td></tr>",
-      monitornames[s->monitor]);
-
-    for(d= s->dependantlist; d; d= d->next) {
-      if(d->dependant != NULL) {
-        out_print(res,
-          "<tr><td>Depends on service </td><td> <a href=%s> %s </a></td></tr>",
-          d->dependant, d->dependant);
-      }
+        "<tr><td>Depends on service </td><td> <a href=%s> %s </a></td></tr>",
+        d->dependant, d->dependant);
     }
+  }
 
-    if(s->start) {
-      int i= 0;
-      out_print(res, "<tr><td>Start program</td><td>'");
-      while(s->start->arg[i]) {
-        if(i) out_print(res, " ");
-        out_print(res, "%s", s->start->arg[i++]);
-      }
-      out_print(res, "'");
-      if(s->start->has_uid)
-        out_print(res, " as uid %d", s->start->uid);
-      if(s->start->has_gid)
-        out_print(res, " as gid %d", s->start->gid);
-      out_print(res, " timeout %d second(s)", s->start->timeout);
-      out_print(res, "</td></tr>");
+  if(s->start) {
+    int i= 0;
+    out_print(res, "<tr><td>Start program</td><td>'");
+    while(s->start->arg[i]) {
+      if(i) out_print(res, " ");
+      out_print(res, "%s", s->start->arg[i++]);
     }
+    out_print(res, "'");
+    if(s->start->has_uid)
+      out_print(res, " as uid %d", s->start->uid);
+    if(s->start->has_gid)
+      out_print(res, " as gid %d", s->start->gid);
+    out_print(res, " timeout %d second(s)", s->start->timeout);
+    out_print(res, "</td></tr>");
+  }
 
-    if(s->stop) {
-      int i= 0;
-      out_print(res, "<tr><td>Stop program</td><td>'");
-      while(s->stop->arg[i]) {
-        if(i) out_print(res, " ");
-        out_print(res, "%s", s->stop->arg[i++]);
-      }
-      out_print(res, "'");
-      if(s->stop->has_uid)
-        out_print(res, " as uid %d", s->stop->uid);
-      if(s->stop->has_gid)
-        out_print(res, " as gid %d", s->stop->gid);
-      out_print(res, " timeout %d second(s)", s->stop->timeout);
-      out_print(res, "</td></tr>");
+  if(s->stop) {
+    int i= 0;
+    out_print(res, "<tr><td>Stop program</td><td>'");
+    while(s->stop->arg[i]) {
+      if(i) out_print(res, " ");
+      out_print(res, "%s", s->stop->arg[i++]);
     }
+    out_print(res, "'");
+    if(s->stop->has_uid)
+      out_print(res, " as uid %d", s->stop->uid);
+    if(s->stop->has_gid)
+      out_print(res, " as gid %d", s->stop->gid);
+    out_print(res, " timeout %d second(s)", s->stop->timeout);
+    out_print(res, "</td></tr>");
+  }
 
-    out_print(res,
-      "<tr><td>Check service</td><td>every %d cycle</td></tr>",
-      s->every?s->every:1);
+  out_print(res,
+    "<tr><td>Check service</td><td>every %d cycle</td></tr>",
+    s->every?s->every:1);
 
-    if(s->def_timeout && s->action_TIMEOUT) {
-      EventAction_T a= s->action_TIMEOUT;
-      out_print(res,
-        "<tr><td>Timeout</td>"
-        "<td>If %d restart within %d cycles then %s else if succeeded "
-        "then %s</td></tr>",
-         s->to_start,
-         s->to_cycle,
-         a->failed->description,
-         a->succeeded->description);
-    }
+  for (ar = s->actionratelist; ar; ar = ar->next)
+    out_print(res, "<tr><td>Timeout</td><td>If restarted %d times within %d cycle(s) then %s</td></tr>", ar->count, ar->cycle, ar->action->failed->description);
 
-    ctime_r((const time_t *)&s->collected.tv_sec, time);
-    out_print(res,
-      "<tr><td>Data collected</td><td>%s</td></tr>",
-      time);
+  ctime_r((const time_t *)&s->collected.tv_sec, time);
+  out_print(res,
+    "<tr><td>Data collected</td><td>%s</td></tr>",
+    time);
 
-    /* Parameters */
-    print_service_params_icmp(res, s);
-    print_service_params_port(res, s);
-    print_service_params_perm(res, s);
-    print_service_params_uid(res, s);
-    print_service_params_gid(res, s);
-    print_service_params_timestamp(res, s);
-    print_service_params_filesystem(res, s);
-    print_service_params_size(res, s);
-    print_service_params_match(res, s);
-    print_service_params_checksum(res, s);
-    print_service_params_process(res, s);
-    print_service_params_resource(res, s);
+  /* Parameters */
+  print_service_params_icmp(res, s);
+  print_service_params_port(res, s);
+  print_service_params_perm(res, s);
+  print_service_params_uid(res, s);
+  print_service_params_gid(res, s);
+  print_service_params_timestamp(res, s);
+  print_service_params_filesystem(res, s);
+  print_service_params_size(res, s);
+  print_service_params_match(res, s);
+  print_service_params_checksum(res, s);
+  print_service_params_process(res, s);
+  print_service_params_resource(res, s);
 
-    /* Rules */
-    print_service_rules_icmp(res, s);
-    print_service_rules_port(res, s);
-    print_service_rules_perm(res, s);
-    print_service_rules_uid(res, s);
-    print_service_rules_gid(res, s);
-    print_service_rules_timestamp(res, s);
-    print_service_rules_filesystem(res, s);
-    print_service_rules_size(res, s);
-    print_service_rules_match(res, s);
-    print_service_rules_checksum(res, s);
-    print_service_rules_process(res, s);
-    print_service_rules_resource(res, s);
+  /* Rules */
+  print_service_rules_icmp(res, s);
+  print_service_rules_port(res, s);
+  print_service_rules_perm(res, s);
+  print_service_rules_uid(res, s);
+  print_service_rules_gid(res, s);
+  print_service_rules_timestamp(res, s);
+  print_service_rules_filesystem(res, s);
+  print_service_rules_size(res, s);
+  print_service_rules_match(res, s);
+  print_service_rules_checksum(res, s);
+  print_service_rules_process(res, s);
+  print_service_rules_resource(res, s);
 
-    print_alerts(res, s->maillist);
+  print_alerts(res, s->maillist);
 
-    out_print(res, "</table>");
+  out_print(res, "</table>");
 
-    print_buttons(req, res, s);
-
-    FOOT
-
-  END_LOCK;
+  print_buttons(req, res, s);
 
+  FOOT
 }
 
 
@@ -2055,7 +2028,7 @@ static void print_service_params_port(HttpResponse res, Service_T s) {
         if(p->family == AF_INET) {
           if(!p->is_available) {
             out_print(res,
-              "<tr><td>Port Response time</font></td><td>"
+              "<tr><td>Port Response time</td><td>"
               "<font color='#ff0000'>connection failed to %s:%d%s [%s via %s]</font>"
               "</td></tr>",
               p->hostname, p->port, p->request?p->request:"",
@@ -2070,7 +2043,7 @@ static void print_service_params_port(HttpResponse res, Service_T s) {
         } else if(p->family == AF_UNIX) {
           if(!p->is_available) {
             out_print(res,
-              "<tr><td>Unix Socket Response time</font></td><td>"
+              "<tr><td>Unix Socket Response time</td><td>"
               "<font color='#ff0000'>connection failed to %s [%s]</font>"
               "</td></tr>",
               p->pathname, p->protocol->name);
@@ -2102,13 +2075,9 @@ static void print_service_params_icmp(HttpResponse res, Service_T s) {
 
       for(i= s->icmplist; i; i= i->next) {
         if(!i->is_available) {
-          out_print(res,
-            "<tr><td>ICMP Response time</font></td><td><font color='#ff0000'>"
-            "connection failed [%s]</font></td></tr>", icmpnames[i->type]);
+          out_print(res, "<tr><td>ICMP Response time</td><td><font color='#ff0000'>connection failed [%s]</font></td></tr>", icmpnames[i->type]);
         } else {
-          out_print(res,
-            "<tr><td>ICMP Response time</td><td>%.3fs [%s]</td></tr>",
-            i->response, icmpnames[i->type]);
+          out_print(res, "<tr><td>ICMP Response time</td><td>%.3fs [%s]</td></tr>", i->response, icmpnames[i->type]);
         }
       }
     }
@@ -2125,7 +2094,7 @@ static void print_service_params_perm(HttpResponse res, Service_T s) {
 
     if(!Util_hasServiceStatus(s)) {
 
-      out_print(res, "<tr><td>Permission</td><td>-</font></td></tr>");
+      out_print(res, "<tr><td>Permission</td><td>-</td></tr>");
 
     } else {
 
@@ -2148,7 +2117,7 @@ static void print_service_params_uid(HttpResponse res, Service_T s) {
 
     if(!Util_hasServiceStatus(s)) {
 
-      out_print(res, "<tr><td>UID</td><td>-</font></td></tr>");
+      out_print(res, "<tr><td>UID</td><td>-</td></tr>");
 
     } else {
 
@@ -2171,7 +2140,7 @@ static void print_service_params_gid(HttpResponse res, Service_T s) {
 
     if(!Util_hasServiceStatus(s)) {
 
-      out_print(res, "<tr><td>GID</td><td>-</font></td></tr>");
+      out_print(res, "<tr><td>GID</td><td>-</td></tr>");
 
     } else {
 
@@ -2193,7 +2162,7 @@ static void print_service_params_timestamp(HttpResponse res, Service_T s) {
 
     if(!Util_hasServiceStatus(s)) {
 
-      out_print(res, "<tr><td>Timestamp</td><td>-</font></td></tr>");
+      out_print(res, "<tr><td>Timestamp</td><td>-</td></tr>");
 
     } else {
 
@@ -2278,7 +2247,7 @@ static void print_service_params_size(HttpResponse res, Service_T s) {
     if(!Util_hasServiceStatus(s)) {
 
       out_print(res,
-        "<tr><td>Size</td><td>-</font></td></tr>");
+        "<tr><td>Size</td><td>-</td></tr>");
 
     } else {
 
@@ -2298,7 +2267,7 @@ static void print_service_params_match(HttpResponse res, Service_T s) {
     if(!Util_hasServiceStatus(s)) {
 
       out_print(res,
-        "<tr><td>Match regex</td><td>-</font></td></tr>");
+        "<tr><td>Match regex</td><td>-</td></tr>");
 
     } else {
 
@@ -2317,7 +2286,7 @@ static void print_service_params_checksum(HttpResponse res, Service_T s) {
 
     if(!Util_hasServiceStatus(s)) {
 
-      out_print(res, "<tr><td>Checksum</td><td>-</font></td></tr>");
+      out_print(res, "<tr><td>Checksum</td><td>-</td></tr>");
 
     } else {
 
