@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2000-2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -218,6 +218,7 @@
 #define ICMP_ATTEMPT_COUNT      3         
 #define ICMP_MAX_COUNT          20
 
+
 /** ------------------------------------------------- Special purpose macros */
 
 
@@ -272,6 +273,14 @@ Sigfunc *signal(int signo, Sigfunc * func);
 typedef char MD_T[MD_SIZE];
 
 
+/** Defines an string buffer object */
+typedef struct mybuffer {
+  char          *buf;                               /**< String buffer       */
+  size_t         bufsize;                           /**< String buffer size  */
+  size_t         bufused;                           /**< String buffer usage */
+} Buffer_T;
+
+
 /**
  * Defines a Command with ARGMAX optional arguments. The arguments
  * array must be NULL terminated and the first entry is the program
@@ -337,7 +346,6 @@ typedef struct mymmonit {
   URL_T url;                                             /**< URL definition */
   Ssl_T ssl;                                             /**< SSL definition */
   int   timeout;              /**< The timeout to wait for connection or i/o */
-  Socket_T       socket;                     /**< Socket used for connection */
 
   /** For internal use */
   struct mymmonit *next;                         /**< next receiver in chain */
@@ -401,8 +409,8 @@ typedef struct myprocesstree {
   long  cputime;                                           /**< 1/10 seconds */
   long  cputime_prev;                                      /**< 1/10 seconds */
   
-  struct myprocesstree  *parent;
-  struct myprocesstree **children;
+  int parent;
+  int *children;
 } ProcessTree_T;
 
 
@@ -449,6 +457,7 @@ typedef struct myport {
   int  port;                                                 /**< Portnumber */
   char *request;                              /**< Specific protocol request */
   char *request_checksum;     /**< The optional checksum for a req. document */
+  char *request_hostheader;            /**< The optional Host: header to use */
   int  request_hashtype;  /**< The optional type of hash for a req. document */
   char *pathname;                   /**< Pathname, in case of an UNIX socket */
   char *address;               /**< Human readable destination of the socket */
@@ -505,11 +514,28 @@ typedef struct myicmp {
 } *Icmp_T;
 
 
-typedef struct mydependant {
-  char *dependant;                           /**< name of dependant service  */
+typedef struct myservicegroupmember {
+  char *name;                                           /**< name of service */
 
   /** For internal use */
-  struct mydependant *next;           /**< next dependant service in chain   */
+  struct myservicegroupmember *next;              /**< next service in chain */
+} *ServiceGroupMember_T;
+
+
+typedef struct myservicegroup {
+  char *name;                                     /**< name of service group */
+  struct myservicegroupmember *members;           /**< Service group members */
+
+  /** For internal use */
+  struct myservicegroup *next;              /**< next service group in chain */
+} *ServiceGroup_T;
+
+
+typedef struct mydependant {
+  char *dependant;                            /**< name of dependant service */
+
+  /** For internal use */
+  struct mydependant *next;             /**< next dependant service in chain */
 } *Dependant_T;
 
 
@@ -538,6 +564,17 @@ typedef struct mytimestamp {
 } *Timestamp_T;
 
 
+/** Defines action rate object */
+typedef struct myactionrate {
+  int  count;                                            /**< Action counter */
+  int  cycle;                                             /**< Cycle counter */
+  EventAction_T action;    /**< Description of the action upon matching rate */
+ 
+  /** For internal use */
+  struct myactionrate *next;                   /**< next actionrate in chain */
+} *ActionRate_T;
+
+
 /** Defines size object */
 typedef struct mysize {
   int  operator;                                    /**< Comparison operator */
@@ -664,18 +701,14 @@ typedef struct myservice {
 
   /** Common parameters */
   char *name;                                  /**< Service descriptive name */
-  char *group;                                       /**< Service group name */
   int (*check)(struct myservice *);       /**< Service verification function */
   int  type;                                     /**< Monitored service type */
   int  monitor;                                      /**< Monitor state flag */
   int  mode;                            /**< Monitoring mode for the service */
   int  ncycle;                          /**< The number of the current cycle */
   int  nstart;           /**< The number of current starts with this service */
-  int  to_start;                                  /**< Timeout start ceiling */
-  int  to_cycle;                                  /**< Timeout cycle ceiling */
   int  every;                        /**< Check this program at given cycles */
   int  nevery;          /**< Counter for every.  When nevery == every, check */
-  int  def_timeout;          /**< TRUE if timeout is defined for the service */
   int  def_every;              /**< TRUE if every is defined for the service */
   int  visited;      /**< Service visited flag, set if dependencies are used */
   int  depend_visited;/**< Depend visited flag, set if dependencies are used */
@@ -686,6 +719,7 @@ typedef struct myservice {
   Mail_T      maillist;                  /**< Alert notification mailinglist */
 
   /** Test rules and event handlers */
+  ActionRate_T actionratelist;                    /**< ActionRate check list */
   Checksum_T  checksum;                                  /**< Checksum check */
   Filesystem_T filesystemlist;                    /**< Filesystem check list */
   Gid_T       gid;                                            /**< Gid check */
@@ -707,7 +741,6 @@ typedef struct myservice {
   EventAction_T action_EXEC;       /**< Description of the action upon event */
   EventAction_T action_INVALID;    /**< Description of the action upon event */
   EventAction_T action_NONEXIST;   /**< Description of the action upon event */
-  EventAction_T action_TIMEOUT;    /**< Description of the action upon event */
 
   /** Internal monit events */
   EventAction_T action_MONIT_START;         /**< Monit instance start action */
@@ -725,11 +758,10 @@ typedef struct myservice {
 
   /** Events */
   struct myevent {
-    #define           EVENT_VERSION  2      /**< The event structure version */
+    #define           EVENT_VERSION  3      /**< The event structure version */
     int               id;                      /**< The event identification */
     struct timeval    collected;                 /**< When the event occured */
     char             *source;                 /**< Event source service name */
-    char             *group;                         /**< Service group name */
     int               mode;             /**< Monitoring mode for the service */
     int               type;                      /**< Monitored service type */
     short             state;         /**< TRUE if failed, FALSE if succeeded */
@@ -824,18 +856,17 @@ struct myrun {
   } MailFormat;
                                           
   pthread_mutex_t mutex;    /**< Mutex used for service data synchronization */
-
 };
 
 
 /* -------------------------------------------------------- Global variables */
 
-char   *prog;                                  /**< The Name of this Program */
-struct myrun Run;                      /**< Struct holding runtime constants */
-Service_T servicelist;                /**< The service list (created in p.y) */
-Service_T servicelist_conf;   /**< The service list in conf file (c. in p.y) */
-SystemInfo_T systeminfo;                              /**< System infomation */
-
+extern char          *prog;
+extern struct myrun   Run;
+extern Service_T      servicelist;
+extern Service_T      servicelist_conf;
+extern ServiceGroup_T servicegrouplist;
+extern SystemInfo_T   systeminfo;
 extern ProcessTree_T *ptree;     
 extern int            ptreesize;    
 extern ProcessTree_T *oldptree;  
@@ -849,14 +880,14 @@ extern int            oldptreesize;
 /* FIXME: move remaining prototypes into seperate header-files */
 
 int   parse(char *);
-void  control_service(const char *, int);
-void  control_service_string(const char *, const char *);
-void  control_service_daemon(const char *, const char *);
+int   control_service(const char *, int);
+int   control_service_string(const char *, const char *);
+int   control_service_daemon(const char *, const char *);
 void  setup_dependants();
 void  reset_depend();
 void  spawn(Service_T, Command_T, const char *);
 int   status(char *);
-int   log_init() ;
+int   log_init();
 void  LogEmergency(const char *, ...);
 void  LogAlert(const char *, ...);
 void  LogCritical(const char *, ...);
@@ -871,8 +902,8 @@ void  log_close();
 void vsyslog (int, const char *, va_list);
 #endif /* HAVE_SYSLOG */
 #endif /* HAVE_VSYSLOG */
-void  validate() ;
-void  daemonize() ;
+int   validate();
+void  daemonize();
 void  gc();
 void  gc_mail_list(Mail_T *);
 void  gccmd(Command_T *);
