@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -144,8 +144,6 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.c,v 1.83 2009/05/04 19:53:48 martinp Exp $
- *
  *  @file
  */
 
@@ -240,24 +238,29 @@ int check_udp_socket(int socket) {
 int create_socket(const char *hostname, int port, int type, int timeout) {
 
   int s;
-  struct hostent *hp;
   struct sockaddr_in sin;
+  struct sockaddr_in *sa;
+  struct addrinfo hints;
+  struct addrinfo *result;
   
   ASSERT(hostname);
 
-  if((hp= gethostbyname(hostname)) == NULL) {
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = AF_INET;
+  if(getaddrinfo(hostname, NULL, &hints, &result) != 0) {
     return -1;
   }
 
-  endhostent();
-  
   if((s= socket(AF_INET, type, 0)) < 0) {
+    freeaddrinfo(result);
     return -1;
   }
 
+  sa = (struct sockaddr_in *)result->ai_addr;
+  memcpy(&sin, sa, result->ai_addrlen);
   sin.sin_family= AF_INET;
   sin.sin_port= htons(port);
-  memcpy(&sin.sin_addr, hp->h_addr, hp->h_length);
+  freeaddrinfo(result);
   
   if(! set_noblock(s)) {
     goto error;
@@ -321,6 +324,7 @@ int create_unix_socket(const char *pathname, int timeout) {
   ASSERT(pathname);
 
   if((s= socket(PF_UNIX, SOCK_STREAM, 0)) < 0) {
+    LogError("%s: Cannot create socket -- %s\n", prog, STRERROR);
     return -1;
   }
 
@@ -331,8 +335,7 @@ int create_unix_socket(const char *pathname, int timeout) {
     goto error;
   }
   
-  if(do_connect(s, (struct sockaddr *)&unixsocket, sizeof(unixsocket),
-		timeout) < 0) {
+  if(do_connect(s, (struct sockaddr *)&unixsocket, sizeof(unixsocket), timeout) < 0) {
     goto error;
   }
   
@@ -358,46 +361,60 @@ int create_unix_socket(const char *pathname, int timeout) {
  * @return The socket ready for accept, or -1 if an error occured.
  */
 int create_server_socket(int port, int backlog, const char *bindAddr) {
-  
   int s;
+  int status;
   int flag= 1;
   struct sockaddr_in myaddr;
 
   if((s= socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+    LogError("%s: Cannot create socket -- %s\n", prog, STRERROR);
     return -1;
   }
 
   memset(&myaddr, 0, sizeof(struct sockaddr_in));
-  myaddr.sin_family= AF_INET;
-  myaddr.sin_port= htons(port);
   
   if(bindAddr) {
-    struct hostent *h= gethostbyname(bindAddr);
-    if(h==NULL) {
-      errno= h_errno;
+    struct sockaddr_in *sa;
+    struct addrinfo hints;
+    struct addrinfo *result;    
+
+    memset(&hints, 0, sizeof(struct addrinfo));
+    hints.ai_family = AF_INET;
+    if((status = getaddrinfo(bindAddr, NULL, &hints, &result)) != 0) {
+      LogError("%s: Cannot translate '%s' to IP address -- %s\n", prog, bindAddr, gai_strerror(status));
       goto error;
     }
-    myaddr.sin_addr= *(struct in_addr*)h->h_addr_list[0];
+    sa = (struct sockaddr_in *)result->ai_addr;
+    memcpy(&myaddr, sa, result->ai_addrlen);
+    freeaddrinfo(result);
   } else {
     myaddr.sin_addr.s_addr= htonl(INADDR_ANY);
   }
+  myaddr.sin_family= AF_INET;
+  myaddr.sin_port= htons(port);
 
-  endhostent();
-
-  if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&flag, sizeof(flag)) < 0) 
+  if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&flag, sizeof(flag)) < 0)  {
+    LogError("%s: Cannot set reuseaddr option -- %s\n", prog, STRERROR);
     goto error;
+  }
   
   if(! set_noblock(s))
     goto error;
   
-  if(fcntl(s, F_SETFD, FD_CLOEXEC) == -1)
+  if(fcntl(s, F_SETFD, FD_CLOEXEC) == -1) {
+    LogError("%s: Cannot set close on exec option -- %s\n", prog, STRERROR);
     goto error; 
+  }
   
-  if(bind(s, (struct sockaddr *)&myaddr, sizeof(struct sockaddr_in)) < 0)
+  if(bind(s, (struct sockaddr *)&myaddr, sizeof(struct sockaddr_in)) < 0) {
+    LogError("%s: Cannot bind -- %s\n", prog, STRERROR);
     goto error;
+  }
   
-  if(listen(s, backlog) < 0)
+  if(listen(s, backlog) < 0) {
+    LogError("%s: Cannot listen -- %s\n", prog, STRERROR);
     goto error;
+  }
   
   return s;
 
@@ -435,14 +452,13 @@ int close_socket(int socket) {
  * @return TRUE if success, otherwise FALSE
  */
 int set_noblock(int socket) {
-  
-  int flags;
-
-  flags= fcntl(socket, F_GETFL, 0);
-  flags |= O_NONBLOCK;
-
-  return (fcntl(socket, F_SETFL, flags) == 0);
+  int flags = fcntl(socket, F_GETFL, 0);
 
+  if (fcntl(socket, F_SETFL, flags|O_NONBLOCK) == -1) {
+    LogError("%s: Cannot set nonblocking -- %s\n", prog, STRERROR);
+    return FALSE;
+  }
+  return TRUE;
 }
 
 
@@ -637,10 +653,11 @@ int udp_write(int socket, void *b, int len, int timeout) {
  * @return response time on succes, -1 on error
  */
 double icmp_echo(const char *hostname, int timeout, int count) {
-
-  struct hostent *hp;
   struct sockaddr_in sin;
   struct sockaddr_in sout;
+  struct sockaddr_in *sa;
+  struct addrinfo hints;
+  struct addrinfo *result;
 #ifdef HAVE_SOL_IP
   struct iphdr *iphdrin;
   struct icmphdr *icmphdrin= NULL;
@@ -660,18 +677,20 @@ double icmp_echo(const char *hostname, int timeout, int count) {
   struct timeval tv;
   struct timeval t1[ICMP_MAX_COUNT];
   struct timeval t2;
-  double response= -1;
+  double response= -1.;
   
   ASSERT(hostname);   
   ASSERT(count <= ICMP_MAX_COUNT);
 
-  if((hp= gethostbyname(hostname)) == NULL)
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = AF_INET;
+  if(getaddrinfo(hostname, NULL, &hints, &result) != 0)
     return response;
 
-  endhostent();
-
-  if((s= socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0)
+  if((s= socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0) {
+    freeaddrinfo(result);
     return response;
+  }
 
 #ifdef HAVE_SOL_IP
   sol_ip = SOL_IP;
@@ -683,8 +702,10 @@ double icmp_echo(const char *hostname, int timeout, int count) {
   }
 #endif
 
-  if(setsockopt(s, sol_ip, IP_TTL, (char *)&ttl, sizeof(ttl)) < 0)
+  if(setsockopt(s, sol_ip, IP_TTL, (char *)&ttl, sizeof(ttl)) < 0) {
+    freeaddrinfo(result);
     goto error2;
+  }
 
   tv.tv_sec= timeout;
   tv.tv_usec= 0;
@@ -704,9 +725,10 @@ double icmp_echo(const char *hostname, int timeout, int count) {
     icmphdrout[i]->icmp_seq= i;
     icmphdrout[i]->icmp_cksum= checksum_ip((unsigned char *)icmphdrout[i], ICMP_SIZE);
 #endif
+    sa = (struct sockaddr_in *)result->ai_addr;
+    memcpy(&sout, sa, result->ai_addrlen);
     sout.sin_family= AF_INET;
     sout.sin_port= 0;
-    memcpy(&sout.sin_addr, hp->h_addr, hp->h_length);
 
     /* Get time of particular connection attempt beginning */
     gettimeofday(&t1[i], NULL);
@@ -716,6 +738,7 @@ double icmp_echo(const char *hostname, int timeout, int count) {
 	      (struct sockaddr *)&sout, sizeof(struct sockaddr));
     } while(n == -1 && errno == EINTR);
   }
+  freeaddrinfo(result);
   
   do {
 
