@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -46,6 +46,10 @@
 #include <kvm.h>
 #endif
 
+#ifdef HAVE_PATHS_H
+#include <paths.h>
+#endif
+
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -80,9 +84,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Rory Toma <rory@digeo.com>
- *  @author Martin Pala<martinp@tildeslash.com>
- *
- *  @version \$Id: sysdep_FREEBSD.c,v 1.41 2009/02/13 09:18:16 hauk Exp $
+ *  @author Martin Pala <martinp@tildeslash.com>
  *
  *  @file
  */
@@ -109,27 +111,23 @@ int init_process_info_sysdep(void) {
   mib[0] = CTL_KERN;
   mib[1] = KERN_CLOCKRATE;
   len    = sizeof(clock);
-  if(sysctl(mib, 2, &clock, &len, NULL, 0) == -1)
-  {
+  if (sysctl(mib, 2, &clock, &len, NULL, 0) == -1) {
     DEBUG("system statistic error -- cannot get clock rate: %s\n", STRERROR);
     return FALSE;
   }
   hz     = clock.hz;
 
   mib[0] = CTL_HW;
-
   mib[1] = HW_NCPU;
   len    = sizeof(systeminfo.cpus);
-  if(sysctl(mib, 2, &systeminfo.cpus, &len, NULL, 0) == -1)
-  {
+  if (sysctl(mib, 2, &systeminfo.cpus, &len, NULL, 0) == -1) {
     DEBUG("system statistic error -- cannot get cpu count: %s\n", STRERROR);
     return FALSE;
   }
 
   mib[1] = HW_PHYSMEM;
   len    = sizeof(systeminfo.mem_kbyte_max);
-  if(sysctl(mib, 2, &systeminfo.mem_kbyte_max, &len, NULL, 0) == -1)
-  {
+  if (sysctl(mib, 2, &systeminfo.mem_kbyte_max, &len, NULL, 0) == -1) {
     DEBUG("system statistic error -- cannot get real memory amount: %s\n", STRERROR);
     return FALSE;
   }
@@ -137,8 +135,7 @@ int init_process_info_sysdep(void) {
 
   mib[1] = HW_PAGESIZE;
   len    = sizeof(pagesize_kbyte);
-  if(sysctl(mib, 2, &pagesize_kbyte, &len, NULL, 0) == -1)
-  {
+  if (sysctl(mib, 2, &pagesize_kbyte, &len, NULL, 0) == -1) {
     DEBUG("system statistic error -- cannot get memory page size: %s\n", STRERROR);
     return FALSE;
   }
@@ -160,20 +157,13 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
   ProcessTree_T     *pt;
   struct kinfo_proc *pinfo;
 
-  if(getuid()!=0) {
-    LogError("system statistic error -- permission denied\n");
-    return FALSE;
-  }
-
-  if(!(kvm_handle = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL)))
-  {
+  if (!(kvm_handle = kvm_open(NULL, _PATH_DEVNULL, NULL, O_RDONLY, prog))) {
     LogError("system statistic error -- cannot initialize kvm interface\n");
     return FALSE;
   }
 
   pinfo = kvm_getprocs(kvm_handle, KERN_PROC_ALL, 0, &treesize);
-  if(!pinfo || (treesize < 1))
-  {
+  if (!pinfo || (treesize < 1)) {
     LogError("system statistic error -- cannot get process tree\n");
     kvm_close(kvm_handle);
     return FALSE;
@@ -181,24 +171,21 @@ int initprocesstree_sysdep(ProcessTree_T **reference) {
 
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
 
-  for(i = 0; i < treesize; i++)
-  {
-   #if(__FreeBSD_version > 500000)
+  for (i = 0; i < treesize; i++) {
+   #if (__FreeBSD_version > 500000)
     pt[i].pid       = pinfo[i].ki_pid;
     pt[i].ppid      = pinfo[i].ki_ppid;
     pt[i].cputime   = (long)(pinfo[i].ki_runtime / 100000);
     pt[i].mem_kbyte = (unsigned long)(pinfo[i].ki_rssize * pagesize_kbyte);
-    if(pinfo[i].ki_stat == SZOMB)
+    if (pinfo[i].ki_stat == SZOMB)
    #else
     pt[i].pid       = pinfo[i].kp_proc.p_pid;
     pt[i].ppid      = pinfo[i].kp_eproc.e_ppid;
     pt[i].cputime   = (long)(pinfo[i].kp_proc.p_runtime / 100000);
     pt[i].mem_kbyte = (unsigned long)(pinfo[i].kp_eproc.e_vm.vm_rssize * pagesize_kbyte);
-    if(pinfo[i].kp_proc.p_stat == SZOMB)
+    if (pinfo[i].kp_proc.p_stat == SZOMB)
    #endif
-    {
       pt[i].status_flag |= PROCESS_ZOMBIE;
-    }
     pt[i].cpu_percent = 0;
     pt[i].time = get_float_time();
   }
@@ -231,10 +218,8 @@ int used_system_memory_sysdep(SystemInfo_T *si) {
   size_t         len   = sizeof(struct vmtotal);
   struct vmtotal vm;
 
-  if(sysctl(mib, 2, &vm, &len, NULL, 0) == -1)
-  {
-    LogError("system statistic error -- cannot get real memory usage: %s\n",
-      STRERROR);
+  if (sysctl(mib, 2, &vm, &len, NULL, 0) == -1) {
+    LogError("system statistic error -- cannot get real memory usage: %s\n", STRERROR);
     return FALSE;
   }
 
@@ -256,33 +241,26 @@ int used_system_cpu_sysdep(SystemInfo_T *si) {
   size_t len;
 
   len = sizeof(mib);
-  if(sysctlnametomib("kern.cp_time", mib, &len) == -1)
-  {
-    LogError("system statistic error -- cannot get cpu time handler: %s\n",
-      STRERROR);
+  if (sysctlnametomib("kern.cp_time", mib, &len) == -1) {
+    LogError("system statistic error -- cannot get cpu time handler: %s\n", STRERROR);
     return FALSE;
   }
 
   len = sizeof(cp_time);
-  if(sysctl(mib, 2, &cp_time, &len, NULL, 0) == -1)
-  {
+  if (sysctl(mib, 2, &cp_time, &len, NULL, 0) == -1) {
     LogError("system statistic error -- cannot get cpu time: %s\n", STRERROR);
     return FALSE;
   }
 
-  for(i = 0; i < CPUSTATES; i++)
-  {
+  for (i = 0; i < CPUSTATES; i++)
     total_new += cp_time[i];
-  }
+
   total     = total_new - total_old;
   total_old = total_new;
 
-  si->total_cpu_user_percent =
-    (total > 0)?(int)(1000 * (double)(cp_time[CP_USER] - cpu_user_old) / total):-10;
-  si->total_cpu_syst_percent =
-    (total > 0)?(int)(1000 * (double)(cp_time[CP_SYS] - cpu_syst_old) / total):-10;
-  si->total_cpu_wait_percent =
-    0; /* there is no wait statistic available */
+  si->total_cpu_user_percent = (total > 0) ? (int)(1000 * (double)(cp_time[CP_USER] - cpu_user_old) / total) : -10;
+  si->total_cpu_syst_percent = (total > 0) ? (int)(1000 * (double)(cp_time[CP_SYS] - cpu_syst_old) / total) : -10;
+  si->total_cpu_wait_percent = 0; /* there is no wait statistic available */
 
   cpu_user_old = cp_time[CP_USER];
   cpu_syst_old = cp_time[CP_SYS];
