@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -114,45 +114,35 @@ struct pst_status *psall;
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Joe Bryant <JBryant@RiteAid.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
- *  @version \$Id: sysdep_HPUX.c,v 1.35 2009/02/13 09:18:17 hauk Exp $
  *
  *  @file
  */
 
 /*
  * Helpful guide for implematation:
- * "SunOS to HP-UX 9.05 Porting Guide" at
- *    http://www.interex.org/tech/9000/Tech/sun_hpux_port/portguide.html
+ * "SunOS to HP-UX 9.05 Porting Guide" at http://www.interex.org/tech/9000/Tech/sun_hpux_port/portguide.html
  */
 
 int init_process_info_sysdep(void) {
   struct pst_dynamic psd;
   struct pst_static pst;
 
-  if (pstat_getdynamic(&psd,sizeof(psd),(size_t)1,0)!=-1) {
-
+  if (pstat_getdynamic(&psd,sizeof(psd),(size_t)1,0) != -1)
     systeminfo.cpus=psd.psd_proc_cnt;
-
-  } else {
-
+  else
     return FALSE;
 
-  }
-
   if (pstat_getstatic(&pst, sizeof(pst), (size_t) 1, 0) != -1) {
-
     systeminfo.mem_kbyte_max=(unsigned long)(pst.physical_memory * (pst.page_size / 1024)); 
     page_size=pst.page_size;
-
   } else {
-
     return FALSE;
-
   }
 
   return TRUE;
 }
 
+
 /**
  * This routine returns 'na' double precision floats containing
  * the load averages in 'a'; at most 3 values will be returned.
@@ -161,30 +151,21 @@ int init_process_info_sysdep(void) {
  * @return: 0 if successful, -1 if failed (and all load averages are 0).
  */
 int getloadavg_sysdep (double *a, int na) {
-  
   struct pst_dynamic psd;
 	
-  if (pstat_getdynamic(&psd,sizeof(psd),(size_t)1,0)!=-1) {
-    
+  if (pstat_getdynamic(&psd, sizeof(psd), (size_t)1, 0) != -1) {
     switch (na) {
     case 3:
-
-      a[2]=psd.psd_avg_15_min;
+      a[2] = psd.psd_avg_15_min;
 
     case 2:
-
-      a[1]=psd.psd_avg_5_min;
+      a[1] = psd.psd_avg_5_min;
 
     case 1:
-
-      a[0]=psd.psd_avg_1_min;
-
+      a[0] = psd.psd_avg_1_min;
     }
-
   } else {
-
     return FALSE;
-
   }
 
   return TRUE;
@@ -197,61 +178,40 @@ int getloadavg_sysdep (double *a, int na) {
  * @return treesize>0 if succeeded otherwise 0.
  */
 int initprocesstree_sysdep(ProcessTree_T ** reference) {
-
   int            i;
   int            treesize;
   ProcessTree_T *pt;
 
   ASSERT(reference);
 
-  /* Gather process data */
-
   pstat_getdynamic(&pst_dyn, sizeof(struct pst_dynamic), 1, 0);
   nproc = pst_dyn.psd_activeprocs;
 
-  if ((psall = (struct pst_status *) xresize(psall, nproc * sizeof(struct pst_status))) == NULL)
-        return 0;
+  if ((psall = (struct pst_status *)xresize(psall, nproc * sizeof(struct pst_status))) == NULL)
+    return 0;
 
-  if ((treesize=pstat_getproc(psall, sizeof(struct pst_status), nproc , 0))==-1) {
+  if ((treesize = pstat_getproc(psall, sizeof(struct pst_status), nproc , 0)) == -1) {
     LogError("system statistic error 1 -- pstat_getproc failed: %s\n", strerror(errno));
     return 0;
   }
 
-  /* Allocate the tree */
-
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
 
-  /* Inspect data */
-
   for (i = 0; i < treesize; i++) {
-
-    pt[i].pid = psall[i].pst_pid;
-    pt[i].ppid = psall[i].pst_ppid;
-
-    /* get the actual time */
-
-    pt[i].time = get_float_time();
-
-    pt[i].cputime =  psall[i].pst_utime + psall[i].pst_stime * 10;
+    pt[i].pid         = psall[i].pst_pid;
+    pt[i].ppid        = psall[i].pst_ppid;
+    pt[i].time        = get_float_time();
+    pt[i].cputime     =  psall[i].pst_utime + psall[i].pst_stime * 10;
     pt[i].cpu_percent = (int)(1000. * psall[i].pst_pctcpu / (float)systeminfo.cpus);
-    pt[i].mem_kbyte = (unsigned long)(psall[i].pst_rssize * (page_size / 1024.0));
-
-    /* State is Zombie -> then we are a Zombie ... clear or? (-: */
-
-    if ( psall[i].pst_stat == PS_ZOMBIE ) {
+    pt[i].mem_kbyte   = (unsigned long)(psall[i].pst_rssize * (page_size / 1024.0));
 
+    if ( psall[i].pst_stat == PS_ZOMBIE )
       pt[i].status_flag |= PROCESS_ZOMBIE;
-      
-    }
-    
   }
 
-  /* Return results */
-
-  * reference = pt;
+  *reference = pt;
 
   return treesize;
-
 }
 
 
@@ -293,21 +253,16 @@ int used_system_cpu_sysdep(SystemInfo_T *si) {
   pstat_getdynamic(&psd, sizeof(psd), 1, 0);
 
   for(i = 0; i < CPUSTATES; i++)
-  {
     cpu_total_new += psd.psd_cpu_time[i];
-  }
   cpu_total     = cpu_total_new - cpu_total_old;
   cpu_total_old = cpu_total_new;
   cpu_user      = psd.psd_cpu_time[CP_USER] + psd.psd_cpu_time[CP_NICE];
   cpu_syst      = psd.psd_cpu_time[CP_SYS];
   cpu_wait      = psd.psd_cpu_time[CP_WAIT];
 
-  si->total_cpu_user_percent =
-    (cpu_total > 0)?(int)(1000 * (double)(cpu_user - cpu_user_old) / cpu_total):-10;
-  si->total_cpu_syst_percent =
-    (cpu_total > 0)?(int)(1000 * (double)(cpu_syst - cpu_syst_old) / cpu_total):-10;
-  si->total_cpu_wait_percent =
-    (cpu_total > 0)?(int)(1000 * (double)(cpu_wait - cpu_wait_old) / cpu_total):-10;
+  si->total_cpu_user_percent = (cpu_total > 0)?(int)(1000 * (double)(cpu_user - cpu_user_old) / cpu_total):-10;
+  si->total_cpu_syst_percent = (cpu_total > 0)?(int)(1000 * (double)(cpu_syst - cpu_syst_old) / cpu_total):-10;
+  si->total_cpu_wait_percent = (cpu_total > 0)?(int)(1000 * (double)(cpu_wait - cpu_wait_old) / cpu_total):-10;
 
   cpu_user_old = cpu_user;
   cpu_syst_old = cpu_syst;
