@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -99,8 +99,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
- *
- *  @version \$Id: sysdep_SOLARIS.c,v 1.29 2009/05/01 16:10:11 martinp Exp $
+ *  @author Martin Pala <martinp@tildeslash.com>
  *
  *  @file
  */
@@ -108,7 +107,6 @@
 #define pagetok(size) ((size) << pageshift)
 
 static int    pageshift=0;
-
 static long   old_cpu_user=0;
 static long   old_cpu_syst=0;
 static long   old_cpu_wait=0;
@@ -119,27 +117,23 @@ static long   old_total=0;
 #endif
 
 int init_process_info_sysdep(void) {
-
   register int pagesize;
 
-  systeminfo.cpus= sysconf( _SC_NPROCESSORS_ONLN);
+  systeminfo.cpus = sysconf( _SC_NPROCESSORS_ONLN);
 
-  pagesize = sysconf(_SC_PAGESIZE);
+  pagesize  = sysconf(_SC_PAGESIZE);
   pageshift = 0;
   while (pagesize > 1) {
-
     pageshift++;
     pagesize >>= 1;
-
   }
 
   /* we only need the amount of log(2)1024 for our conversion */
   pageshift -= LOG1024;
 
-  systeminfo.mem_kbyte_max=pagetok(sysconf(_SC_PHYS_PAGES));
-
-  return (getuid()==0);
+  systeminfo.mem_kbyte_max = pagetok(sysconf(_SC_PHYS_PAGES));
 
+  return (TRUE);
 }
 
 double timestruc_to_tseconds(timestruc_t t) {
@@ -155,96 +149,70 @@ double timestruc_to_tseconds(timestruc_t t) {
  * @return treesize>=0 if succeeded otherwise <0.
  */
 int initprocesstree_sysdep(ProcessTree_T ** reference) {
-
-  char buf[4096];
-  psinfo_t  * psinfo=  (psinfo_t *)&buf;
-  pstatus_t * pstatus= (pstatus_t *)&buf;
-  int      pid;
-  int      i;
-  int      treesize;
-
-  ProcessTree_T *  pt;
-
-  glob_t   globbuf;
+  int            i;
+  int            pid;
+  int            treesize;
+  char           buf[4096];
+  glob_t         globbuf;
+  pstatus_t      pstatus;
+  psinfo_t      *psinfo = (psinfo_t *)&buf;
+  ProcessTree_T *pt;
 
   ASSERT(reference);
 
   /* Find all processes in the /proc directory */
-
-  if (glob("/proc/[0-9]*", 0, NULL, &globbuf) != 0 ) {
-
+  if (glob("/proc/[0-9]*", 0, NULL, &globbuf) != 0)
     return 0;
 
-  } 
-
   treesize = globbuf.gl_pathc;
 
   /* Allocate the tree */
-
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
 
   /* Insert data from /proc directory */
-
-  for ( i = 0; i < treesize; i ++ ) {
-
-    pid=atoi(globbuf.gl_pathv[i]+strlen("/proc/"));
-    pt[i].pid=pid;
+  for (i = 0; i < treesize; i++) {
+    pid = atoi(globbuf.gl_pathv[i] + strlen("/proc/"));
+    pt[i].pid = pid;
 
     /* get the actual time */
     pt[i].time = get_float_time();
 
-    if (!read_proc_file(buf,4096, "psinfo", pt[i].pid)) {
-      
-      pt[i].cputime = 0;
+    if (! read_proc_file(buf,4096, "psinfo", pt[i].pid)) {
+      pt[i].cputime     = 0;
       pt[i].cpu_percent = 0;
-      pt[i].mem_kbyte= 0;
+      pt[i].mem_kbyte   = 0;
       continue;
-
     } 
 
     pt[i].ppid = psinfo->pr_ppid;
         
-    /* If we don't have any light-weight processes (LWP) then we
-       are definitely a zombie */
-
-    if ( psinfo->pr_nlwp == 0 ) {
-
+    /* If we don't have any light-weight processes (LWP) then we are definitely a zombie */
+    if (psinfo->pr_nlwp == 0) {
       pt[i].status_flag = PROCESS_ZOMBIE;
-    
-      /* We can't access /proc/$pid/status of a zombie */
-      /* and does it anyway matter? */
-      
-      pt[i].cputime = 0;
+      pt[i].cputime     = 0;
       pt[i].cpu_percent = 0;
-      pt[i].mem_kbyte= 0;
+      pt[i].mem_kbyte   = 0;
       continue;
-      
     } 
     
     pt[i].mem_kbyte = psinfo->pr_rssize;
     
-    if (!read_proc_file(buf,4096, "status", pt[i].pid)) {
-      
-      pt[i].cputime=0;
+    if (! read_proc_file(buf,4096, "status", pt[i].pid)) {
+      pt[i].cputime     = 0;
       pt[i].cpu_percent = 0;
-      
     } else {
-      
-      pt[i].cputime= ( timestruc_to_tseconds(pstatus->pr_utime) +
-                       timestruc_to_tseconds(pstatus->pr_stime) );
+      memcpy(&pstatus, buf, sizeof(pstatus_t));
+      pt[i].cputime     = (timestruc_to_tseconds(pstatus.pr_utime) + timestruc_to_tseconds(pstatus.pr_stime));
       pt[i].cpu_percent = 0;
     }
-    
   }
   
-  * reference = pt;
+  *reference = pt;
 
   /* Free globbing buffer */
-
   globfree(&globbuf);
 
   return treesize;
-
 }
 
 /**
@@ -255,9 +223,7 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
  * @return: 0 if successful, -1 if failed (and all load averages are 0).
  */
 int getloadavg_sysdep (double *loadv, int nelem) {
-
   return getloadavg(loadv, nelem);
-
 }
 
 
@@ -266,15 +232,12 @@ int getloadavg_sysdep (double *loadv, int nelem) {
  * @return: TRUE if successful, FALSE if failed (or not available)
  */
 int used_system_memory_sysdep(SystemInfo_T *si) {
-
   kstat_ctl_t   *kctl;  
   kstat_named_t *knamed;
   kstat_t       *kstat;
 
-  kctl = kstat_open();
-  
+  kctl  = kstat_open();
   kstat = kstat_lookup(kctl, "unix", 0, "system_pages");
-
   if (kstat_read(kctl, kstat, 0) == -1) {
     LogError("system statistic error -- memory usage gathering failed\n");
     kstat_close(kctl);
@@ -282,10 +245,8 @@ int used_system_memory_sysdep(SystemInfo_T *si) {
   }
   
   knamed = kstat_data_lookup(kstat, "freemem");
-  
-  if (knamed) {
+  if (knamed)
     si->total_mem_kbyte = systeminfo.mem_kbyte_max-pagetok(knamed->value.ul);
-  }
   
   kstat_close(kctl);
 
@@ -298,105 +259,70 @@ int used_system_memory_sysdep(SystemInfo_T *si) {
  * @return: TRUE if successful, FALSE if failed (or not available)
  */
 int used_system_cpu_sysdep(SystemInfo_T *si) {
-  kstat_ctl_t   *kctl;  
-  kstat_named_t *knamed;
-  kstat_t       *kstat;
-
-  kstat_t **cpu_ks;
-  cpu_stat_t *cpu_stat;
-
-  int i, ncpu=0, ncpus;
-  long cpu_user=0, cpu_syst=0, cpu_wait=0;
-  long total=0;
+  int             i, ncpu = 0, ncpus;
+  long            cpu_user = 0, cpu_syst = 0, cpu_wait = 0;
+  long            total = 0;
+  kstat_ctl_t    *kctl;  
+  kstat_named_t  *knamed;
+  kstat_t        *kstat;
+  kstat_t       **cpu_ks;
+  cpu_stat_t     *cpu_stat;
   
-  si->total_cpu_user_percent=0;
-  si->total_cpu_syst_percent=0;
-  si->total_cpu_wait_percent=0;
-
-  kctl = kstat_open();
+  si->total_cpu_user_percent = 0;
+  si->total_cpu_syst_percent = 0;
+  si->total_cpu_wait_percent = 0;
 
+  kctl  = kstat_open();
   kstat = kstat_lookup(kctl, "unix", 0, "system_misc");
-
-  if (kstat_read(kctl, kstat, 0) == -1) {
-
+  if (kstat_read(kctl, kstat, 0) == -1)
     goto error;
-    
-  }
   
-  if (NULL == (knamed = kstat_data_lookup(kstat, "ncpus"))) {
-    
+  if (NULL == (knamed = kstat_data_lookup(kstat, "ncpus")))
     goto error;
-    
-  }
   
   ncpus = knamed->value.ui32;
 
-  cpu_ks = (kstat_t **) xmalloc(ncpus * sizeof (kstat_t *));
-  cpu_stat = (cpu_stat_t *) xmalloc (ncpus * sizeof (cpu_stat_t));
+  cpu_ks   = (kstat_t **)xmalloc(ncpus * sizeof(kstat_t *));
+  cpu_stat = (cpu_stat_t *)xmalloc(ncpus * sizeof(cpu_stat_t));
 
-  for (kstat = kctl->kc_chain; kstat;
-       kstat = kstat->ks_next)
-  {
-    if (strncmp(kstat->ks_name, "cpu_stat", 8) == 0)
-    {
-      if (-1 == kstat_read(kctl, kstat, NULL)) {
+  for (kstat = kctl->kc_chain; kstat; kstat = kstat->ks_next) {
+    if (strncmp(kstat->ks_name, "cpu_stat", 8) == 0) {
 
+      if (-1 == kstat_read(kctl, kstat, NULL))
         goto error2;
-        
-      }
 
       cpu_ks[ncpu] = kstat;
-      ncpu++;
-      if (ncpu > ncpus) {
-        
+      if (++ncpu > ncpus)
         goto error2;
-        
-      }
     }
   }
+  
+  for (i = 0; i < ncpu; i++) {
 
-  
-  
-  for (i = 0; i < ncpu; i++)
-  {
-    if ( -1 == kstat_read(kctl, cpu_ks[i], &cpu_stat[i])) {
-      
+    if (-1 == kstat_read(kctl, cpu_ks[i], &cpu_stat[i]))
       goto error2;
-      
-    }
-    
-    cpu_user+=cpu_stat[i].cpu_sysinfo.cpu[CPU_USER];
-    cpu_syst+=cpu_stat[i].cpu_sysinfo.cpu[CPU_KERNEL];
-    cpu_wait+=cpu_stat[i].cpu_sysinfo.cpu[CPU_WAIT];
-
-    total += ( cpu_stat[i].cpu_sysinfo.cpu[0]+
-               cpu_stat[i].cpu_sysinfo.cpu[1]+
-               cpu_stat[i].cpu_sysinfo.cpu[2]+
-               cpu_stat[i].cpu_sysinfo.cpu[3]);
     
+    cpu_user += cpu_stat[i].cpu_sysinfo.cpu[CPU_USER];
+    cpu_syst += cpu_stat[i].cpu_sysinfo.cpu[CPU_KERNEL];
+    cpu_wait += cpu_stat[i].cpu_sysinfo.cpu[CPU_WAIT];
+    total    += (cpu_stat[i].cpu_sysinfo.cpu[0]+ cpu_stat[i].cpu_sysinfo.cpu[1]+ cpu_stat[i].cpu_sysinfo.cpu[2]+ cpu_stat[i].cpu_sysinfo.cpu[3]);
   }
 
-  if ( old_total == 0.0 ) {
-
-    si->total_cpu_user_percent=-10;
-    si->total_cpu_syst_percent=-10;
-    si->total_cpu_wait_percent=-10;
-    
+  if (old_total == 0.0 ) {
+    si->total_cpu_user_percent = -10;
+    si->total_cpu_syst_percent = -10;
+    si->total_cpu_wait_percent = -10;
   } else {
-
-    si->total_cpu_user_percent=(int)((1000*(cpu_user-old_cpu_user))/
-                                     (total-old_total));
-    si->total_cpu_syst_percent=(int)((1000*(cpu_syst-old_cpu_syst))/
-                                     (total-old_total));
-    si->total_cpu_wait_percent=(int)((1000*(cpu_wait-old_cpu_wait))/
-                                     (total-old_total));
+    si->total_cpu_user_percent = (int)((1000 * (cpu_user - old_cpu_user)) / (total - old_total));
+    si->total_cpu_syst_percent = (int)((1000 * (cpu_syst - old_cpu_syst)) / (total - old_total));
+    si->total_cpu_wait_percent = (int)((1000 * (cpu_wait - old_cpu_wait)) / (total - old_total));
   }
 
   
-  old_cpu_user=cpu_user;
-  old_cpu_syst=cpu_syst;
-  old_cpu_wait=cpu_wait;
-  old_total=total;
+  old_cpu_user = cpu_user;
+  old_cpu_syst = cpu_syst;
+  old_cpu_wait = cpu_wait;
+  old_total    = total;
   
   FREE(cpu_ks);
   FREE(cpu_stat);
@@ -411,5 +337,5 @@ int used_system_cpu_sysdep(SystemInfo_T *si) {
   error:
   kstat_close(kctl);
   return FALSE;
-  
 }
+
