@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -40,18 +40,14 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: sysdep_UNKNOWN.c,v 1.17 2009/02/13 09:18:18 hauk Exp $
- *
  *  @file
  */
 
 
 int init_process_info_sysdep(void) {
-
   systeminfo.mem_kbyte_max = 0;
 
   return FALSE;
-
 }
 
 
@@ -65,9 +61,7 @@ int init_process_info_sysdep(void) {
  * @return treesize>0 if succeeded otherwise =0.
  */
 int initprocesstree_sysdep(ProcessTree_T ** reference) {
-
   return 0;
-
 }
 
 /**
@@ -80,17 +74,12 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
  * @return: 0 if successful, -1 if failed (and all load averages are 0).
  */
 int getloadavg_sysdep (double *loadv, int nelem) {
-
   int i;
   
-  for ( i = 0; i < nelem; i++ ) {
-
+  for (i = 0; i < nelem; i++)
     loadv[i] = 0.0;
 
-  }
-
   return -1;
-
 }
 
 
@@ -99,9 +88,7 @@ int getloadavg_sysdep (double *loadv, int nelem) {
  * @return: TRUE if successful, FALSE if failed (or not available)
  */
 int used_system_memory_sysdep(SystemInfo_T *si) {
-
   return FALSE;
-
 }
 
 
@@ -110,8 +97,6 @@ int used_system_memory_sysdep(SystemInfo_T *si) {
  * @return: TRUE if successful, FALSE if failed (or not available)
  */
 int used_system_cpu_sysdep(SystemInfo_T *si) {
-
   return FALSE;
-  
 }
 
