@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -80,7 +80,6 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
- *  @version \$Id: http.c,v 1.58 2009/02/13 09:18:20 hauk Exp $
  *  @file
  */
 
@@ -112,6 +111,7 @@ int check_http(Socket_T s) {
   char host[STRLEN];
   char auth[STRLEN]= {0};
   const char *request= NULL;
+  const char *hostheader= NULL;
   
   ASSERT(s);
 
@@ -120,6 +120,9 @@ int check_http(Socket_T s) {
   ASSERT(P);
   
   request= P->request?P->request:"/";
+
+  Util_getHTTPHostHeader(s, host, STRLEN);
+  hostheader= P->request_hostheader?P->request_hostheader:host;
  
   if(socket_print(s, 
 		  "GET %s HTTP/1.1\r\n"
@@ -128,8 +131,8 @@ int check_http(Socket_T s) {
 		  "Connection: close\r\n"
 		  "User-Agent: %s/%s\r\n"
 		  "%s\r\n",
-		  request, Util_getHTTPHostHeader(s, host, STRLEN), 
-		  prog, VERSION, get_auth_header(P, auth, STRLEN)) < 0) {
+		  request, hostheader, prog, VERSION,
+                  get_auth_header(P, auth, STRLEN)) < 0) {
     LogError("HTTP: error sending data -- %s\n", STRERROR);
     return FALSE;
   }
