@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -68,6 +68,8 @@ static Protocol_T myrsync= NULL;
 static Protocol_T mytns= NULL;
 static Protocol_T mypgsql= NULL;
 static Protocol_T mysip= NULL;
+static Protocol_T mygps= NULL;
+static Protocol_T myradius= NULL;
 
 
 /**
@@ -76,8 +78,6 @@ static Protocol_T mysip= NULL;
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: protocol.c,v 1.36 2009/02/13 09:18:21 hauk Exp $
- *
  *  @file
  */
 
@@ -111,6 +111,8 @@ void gc_protocols() {
   FREE(mytns);
   FREE(mypgsql);
   FREE(mysip);
+  FREE(mygps);
+  FREE(myradius);
 
 }
 
@@ -351,3 +353,22 @@ void *create_pgsql() {
   return mypgsql;
 }
 
+
+void *create_gps() {
+  if(mygps == NULL) {
+    NEW(mygps);
+    mygps->name= "GPS";
+    mygps->check= check_gps;
+  }
+  return mygps;
+}
+
+void *create_radius() {
+  if(myradius == NULL) {
+    NEW(myradius);
+    myradius->name= "RADIUS";
+    myradius->check= check_radius;
+  }
+  return myradius;
+}
+
