@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -57,8 +57,6 @@ static int expect(Socket_T s, int expect, int log);
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Michael Amster, <mamster@webeasy.com>
  *
- *  @version \$Id: smtp.c,v 1.38 2009/02/13 09:18:22 hauk Exp $
- *
  *  @file
  */
 int check_smtp(Socket_T s) {
@@ -100,22 +98,25 @@ static int say(Socket_T s, char *msg) {
 static int expect(Socket_T s, int expect, int log) {
   
   int status;
+  char text[STRLEN];
   char buf[STRLEN];
 
-  if(!socket_readln(s, buf, STRLEN)) {
-    LogError("SMTP: error receiving data -- %s\n", STRERROR);
-    return FALSE;
+  while (TRUE) {
+    if(!socket_readln(s, buf, STRLEN)) {
+      LogError("SMTP: error receiving data -- %s\n", STRERROR);
+      return FALSE;
+    }
+    Util_chomp(buf);
+    if (sscanf(buf, "%d-%256s", &status, text) != 2) // EHLO send multiple lines
+      break;
   }
-  
-  Util_chomp(buf);
-  
-  sscanf(buf, "%d%*s", &status);
-  if(status != expect) {
+
+  if(sscanf(buf, "%d%*s", &status) != 1 || status != expect) {
     if(log) 
       LogError("SMTP error: %s\n", buf);
     return FALSE;
   }
-  
+
   return TRUE;
   
 }
