@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -155,8 +155,6 @@ static pthread_mutex_t *ssl_mutex_table;
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: ssl.c,v 1.69 2009/05/03 21:29:42 martinp Exp $
- *
  *  @file
  */
 
@@ -351,8 +349,7 @@ ssl_server_connection *init_ssl_server(char *pemfile, char *clientpemfile) {
     goto sslerror;
   }
 
-  if(SSL_CTX_use_certificate_file(ssl_server->ctx, pemfile, 
-				   SSL_FILETYPE_PEM) != 1) {
+  if(SSL_CTX_use_certificate_chain_file(ssl_server->ctx, pemfile) != 1) {
     LogError("%s: Cannot initialize SSL server certificate -- %s\n",
              prog, SSLERROR);
     goto sslerror;
@@ -704,8 +701,7 @@ ssl_connection *new_ssl_connection(char *clientpemfile, int sslversion) {
 
   if(ssl->clientpemfile) {
 
-    if(SSL_CTX_use_certificate_file(ssl->ctx, ssl->clientpemfile, 
-				     SSL_FILETYPE_PEM) <= 0) {
+    if(SSL_CTX_use_certificate_chain_file(ssl->ctx, ssl->clientpemfile) <= 0) {
       LogError("%s: Cannot initialize SSL server certificate -- %s\n",
                prog, SSLERROR);
       goto sslerror;
