@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -74,7 +74,6 @@
  * the default is ~/.monit.state.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @version \$Id: state.c,v 1.39 2009/05/04 20:18:21 martinp Exp $
  *  @file
  */
 
@@ -252,35 +251,24 @@ static FILE *open_state(const char *mode) {
 
 
 static void close_state(FILE *S) {
-
-  if(fclose(S) != 0) {
-    LogCritical("%s: Cannot close the monit state file '%s' -- %s\n",
-	prog, Run.statefile, STRERROR);
-    
-  }
-
+  if(fclose(S) != 0)
+    LogCritical("%s: Cannot close the monit state file '%s' -- %s\n", prog, Run.statefile, STRERROR);
 }
 
 
 static void clone_state(Service_T service, State_T *state) {
-  
   memset(state, 0, sizeof(State_T));
   
   strncpy(state->name, service->name, sizeof(state->name) - 1);
   state->name[sizeof(state->name) - 1] = 0;
-  state->mode= service->mode;
   state->nstart= service->nstart;
   state->ncycle= service->ncycle;
   state->monitor= service->monitor;
-
 }
 
 
 static void update_service_state(Service_T service, State_T *state) {
-
-  service->mode= state->mode;
   service->nstart= state->nstart;
   service->ncycle= state->ncycle;
   service->monitor= state->monitor;
-  
 }
