@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -160,7 +160,6 @@ struct ad_user {
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.c,v 1.241 2009/05/28 21:35:30 martinp Exp $
  *  @file
  */
 
@@ -673,8 +672,8 @@ void Util_printRunList() {
   }
 
   if(Run.mailservers) {
-    printf(" %-18s = ", "Mail server(s)");
     MailServer_T mta= Run.mailservers;
+    printf(" %-18s = ", "Mail server(s)");
     for(mta= Run.mailservers; mta; mta= mta->next)
 	printf("%s:%d%s%s",
           mta->host,
@@ -750,16 +749,19 @@ void Util_printRunList() {
  * @param p A Service_T object
  */
 void Util_printService(Service_T s) {
-  
+  int sgheader = FALSE;
   Port_T n;
   Icmp_T i;
   Mail_T r;
   Filesystem_T dl;
   Resource_T q;
   Timestamp_T t;
+  ActionRate_T ar;
   Size_T sl;
   Match_T ml;
   Dependant_T d;
+  ServiceGroup_T sg;
+  ServiceGroupMember_T sgm;
   char string[STRLEN];
   char ratio1[STRLEN];
   char ratio2[STRLEN];
@@ -768,8 +770,21 @@ void Util_printService(Service_T s) {
  
   snprintf(string, STRLEN, "%s Name", servicetypes[s->type]);
   printf("%-21s = %s\n", string, s->name);
-  if(s->group)
-    printf(" %-20s = %s\n", "Group", s->group);
+
+  for (sg = servicegrouplist; sg; sg = sg->next) {
+    for (sgm = sg->members; sgm; sgm = sgm->next) {
+      if (! strcasecmp(sgm->name, s->name)) {
+        if (! sgheader) {
+          printf(" %-20s = %s", "Group", sg->name);
+          sgheader = TRUE;
+        } else
+          printf(", %s", sg->name);
+      }
+    }
+  }
+  if (sgheader)
+    printf("\n");
+
   if(s->type == TYPE_PROCESS)
     printf(" %-20s = %s\n", "Pid file", s->path);
   else if(s->type != TYPE_HOST && s->type != TYPE_SYSTEM)
@@ -1170,16 +1185,8 @@ void Util_printService(Service_T s) {
   if(s->def_every)
     printf(" %-20s = Check service every %d cycles\n", "Every", s->every);
   
-  if(s->def_timeout && s->action_TIMEOUT) {
-    EventAction_T a= s->action_TIMEOUT;
-    printf(" %-20s = If %d restart within %d cycles then %s "
-      "else if succeeded then %s\n",
-      "Timeout",
-      s->to_start,
-      s->to_cycle,
-      a->failed->description,
-      a->succeeded->description);
-  }
+  for (ar = s->actionratelist; ar; ar = ar->next)
+    printf(" %-20s = If restarted %d times within %d cycle(s) then %s\n", "Timeout", ar->count, ar->cycle, ar->action->failed->description);
 
   for(r= s->maillist; r; r= r->next) {
     printf(" %-20s = %s\n", "Alert mail to", is_str_defined(r->to));
@@ -1362,7 +1369,7 @@ int Util_isProcessRunning(Service_T s) {
   ASSERT(s);
   
   errno= 0;
-  if((pid= Util_getPid(s->path)) >= 0) {
+  if((pid= Util_getPid(s->path)) > 0) {
     if( (getpgid(pid) > -1) || (errno == EPERM) )
       return pid;
     DEBUG("'%s' Error testing process id [%d] -- %s\n", s->name, pid, STRERROR);
@@ -1579,7 +1586,7 @@ char *Util_getBasicAuthHeaderMonit() {
 
   /* We find the first cleartext credential for authorization */
   while (c != NULL) {
-    if (c->digesttype == DIGEST_CLEARTEXT)
+    if (c->digesttype == DIGEST_CLEARTEXT && ! c->is_readonly)
       break;
     c = c->next;
   }
@@ -2180,6 +2187,32 @@ char *Util_portTypeDescription(Port_T p) {
 }
 
 
+void Util_stringbuffer(Buffer_T *b, const char *m, ...) {
+  if (m) {
+    va_list  ap;
+    char    *buf;
+    long     need = 0;
+    ssize_t  have = 0;
+
+    va_start(ap, m);
+    buf = Util_formatString(m, ap, &need);
+    va_end(ap);
+
+    have = (*b).bufsize - (*b).bufused;
+    if (have <= need) {
+      (*b).bufsize += (need + STRLEN);
+      (*b).buf = xresize((*b).buf, (*b).bufsize);
+      if (! (*b).bufused)
+        memset((*b).buf, 0, (*b).bufsize);
+    }
+    memcpy(&(*b).buf[(*b).bufused], buf, need);
+    (*b).bufused += need;
+    (*b).buf[(*b).bufused]= 0;
+    FREE(buf);
+  }
+}
+
+
 /* ----------------------------------------------------------------- Private */
 
 
