@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -40,7 +40,6 @@
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.h,v 1.27 2009/03/11 20:31:27 hauk Exp $
  *  @file
  */
 
@@ -467,4 +466,13 @@ char *Util_getEventratio(Action_T action, char *buf);
 char *Util_portTypeDescription(Port_T p);
 
 
+/**
+ * Print to string buffer
+ * @param b A Buffer object
+ * @param m Format string
+ * @return the socket type description
+ */
+void Util_stringbuffer(Buffer_T *b, const char *m, ...);
+
+
 #endif
